/*
 * This file is part of SionTower.
 *
 *
 * David Saltares Márquez (C) 2011
 * <david.saltares@gmail.com>
 *
 *
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  @file estadoJuego.cpp
 *  @author David Saltares Márquez
 *  @date 24-12-2010
 */

#include <iostream>
#include <cmath>

#include <OGRE/Ogre.h>
#include <boost/bind.hpp>
#include <boost/function.hpp>

#include "stateGame.h"
#include "stateManager.h"
#include "songManager.h"
#include "soundFXManager.h"
#include "collisionManager.h"
#include "levelManager.h"
#include "level.h"
#include "game.h"

using std::cout;
using std::endl;

StateGame::StateGame(StateManager* stateManager): State(stateManager) {
    _log->logMessage("StateGame::StateGame()");

    // Recuperamos el teclado
    _keyboard = _stateManager->getKeyboard();
    // Recuperamos la cámara
    _camera = _sceneManager->getCamera("camera");
    load();
}

StateGame::~StateGame() {
    _log->logMessage("StateGame::~StateGame()");
    clear();
}

void StateGame::load() {
    if (!_loaded) {
        _log->logMessage("StateGame::load()");
        _loaded = true;

        // Luz ambiente
        _sceneManager->setAmbientLight(Ogre::ColourValue(0.25, 0.25, 0.25));


        // Cargamos el personaje
        _player = new Player(_sceneManager, _camera, _keyboard);

        // Cargamos escenario
        _level = LevelManager::getSingleton().getLevel("level01");
        _level->load();
        _song = _level->getSong();
        _song->play();
        _player->setPosition(_level->getPlayerPosition());

        // Callbacks
        CollisionManager::CollisionCallback callback;
        callback = boost::bind(&StateGame::beginCollisionCharacterWall, this, _1, _2);
        CollisionManager::getSingleton().addCollisionCallback(3, 2, callback, CollisionManager::BEGINCOLLISION);
        callback = boost::bind(&StateGame::inCollisionCharacterWall, this, _1, _2);
        CollisionManager::getSingleton().addCollisionCallback(3, 2, callback, CollisionManager::COLLIDING);
        callback = boost::bind(&StateGame::endCollisionCharacterWall, this, _1, _2);
        CollisionManager::getSingleton().addCollisionCallback(3, 2, callback, CollisionManager::ENDCOLLISION);
    }

}

void StateGame::clear() {
    if (_loaded) {
        _log->logMessage("StateGame::limpiar()");
        _loaded = false;

        delete _player;
        _level->unload();
        _sceneManager->clearScene();

        CollisionManager::getSingleton().removeAllBodies();
        CollisionManager::getSingleton().removeAllCollisionCallbacks();
    }
}


void StateGame::update(Ogre::Real deltaT, bool active) {
    if (active) {
        _player->update(deltaT);
        updateCamera(deltaT);
        CollisionManager::getSingleton().checkCollisions();
    }
}


bool StateGame::keyPressed(const OIS::KeyEvent &arg) {
    return true;
}

bool StateGame::keyReleased(const OIS::KeyEvent &arg) {
    return true;
}

bool StateGame::mouseMoved(const OIS::MouseEvent &arg) {
    return true;
}

bool StateGame::mousePressed(const OIS::MouseEvent &arg, OIS::MouseButtonID id) {
    return true;
}

bool StateGame::mouseReleased(const OIS::MouseEvent &arg, OIS::MouseButtonID id) {
    return true;
}


void StateGame::beginCollisionCharacterWall(Body* bodyA, Body* bodyB) {
    cout << "Begin: colisión con muro" << endl;
    _player->restoreOldPosition();
}

void StateGame::inCollisionCharacterWall(Body* bodyA, Body* bodyB) {
    cout << "In: colisión con muro" << endl;
    _player->restoreOldPosition();
}

void StateGame::endCollisionCharacterWall(Body* bodyA, Body* bodyB) {
    cout << "End: colisión con muro" << endl;
}

void StateGame::updateCamera(Ogre::Real deltaT) {
    // Recuperamos objetos
    Ogre::Camera* camera = Game::getCamera();
    OIS::MouseState mouseState = _stateManager->getMouse()->getMouseState();
    Ogre::Viewport* viewport = Game::getViewPort();
    Ogre::Vector3 distance;
    Ogre::Vector3 speed(10, 10, 10);


    // Movimiento lateral
    if (mouseState.X.abs > viewport->getActualWidth() * 0.90) {
        camera->moveRelative(speed * Ogre::Vector3(1, 0, 0) * deltaT);
    }

    if (mouseState.X.abs < viewport->getActualWidth() * 0.10) {
        camera->moveRelative(speed * Ogre::Vector3(-1, 0, 0) * deltaT);
    }

    // Movimiento vertical
    if (mouseState.Y.abs > viewport->getActualHeight() * 0.90) {
        camera->move(speed * Ogre::Vector3(0, 1, 0) * deltaT);
    }

    if (mouseState.Y.abs < viewport->getActualHeight() * 0.10) {
        camera->move(speed * Ogre::Vector3(0, -1, 0) * deltaT);
    }

    Ogre::Vector3 position = camera->getPosition();

    // La cámara no puede bajar más
    if (position.y < 4) {
        position.y = 4;
        camera->setPosition(position);
    }

    // La cámara no puede subir más
    if (position.y > 20) {
        position.y = 20;
        camera->setPosition(position);
    }

    camera->lookAt(0, 0, 0);
}
